SpecStorage = LibStub("AceAddon-3.0"):NewAddon("SpecStorage", "AceConsole-3.0", "AceEvent-3.0" );
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub( "AceLocale-3.0" ):GetLocale( "SpecStorage" )

local SpecStorageFrame
local talentNameToIndex = {}
local glyphNameToIndex = {}
local currentSpecName = nil
local questionMarkTexture = "Interface\\icons\\INV_MISC_QUESTIONMARK"
local addChildNumber = nil
local liveMajorGlyphs = {}

--[[
Frame Layout

SpecStorageFrame
	MainGroup
		Header
		TreeGroup
			ScrollFrame
				Saved Specs
					BossRows
				Add New Spec
					Edit boxes
					Dropdowns
					Add Button
				ButtonGroup
					Load Live Spec
					Clear Selection
]]--

local ldb = LibStub("LibDataBroker-1.1"):NewDataObject("SpecStorage", {
	type = "launcher",
	label = "SpecStorage",
	icon = "Interface\\icons\\spell_nature_enchantarmor",
	OnClick = function(self,button)
		if button == "LeftButton" then
			SpecStorage:Toggle()
		end
	end,
	OnTooltipShow = function(self)
		local cs = "|cffffffcc"
		local ce = "|r"
		self:AddLine("Spec Storage")
		self:AddLine(format("%s"..L["Left-click"].."%s "..L["to toggle"], cs, ce))
	end,
})

local defaults = {
	class  = {
	},
}

function SpecStorage:OnInitialize()
    -- Called when the addon is loaded
	self:RegisterChatCommand("specstorage", "ChatCommand")
	self:RegisterChatCommand("specs", "ChatCommand")
	self.db = LibStub("AceDB-3.0"):New("SpecStorageDB", defaults)
	self:CreateFrame()
end

function SpecStorage:OnEnable()
    -- Called when the addon is enabled
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	self:SetupSpecVariable()
end

function SpecStorage:OnDisable()
    -- Called when the addon is disabled
end

function SpecStorage:ChatCommand(msg)
	if ( msg == "toggle" or msg == "") then
		self:Toggle()
	else
		self:Print(L["Unknown chat command: "]..msg)
	end
end

function SpecStorage:Toggle()
	if SpecStorageFrame:IsVisible() then
		SpecStorageFrame:Hide()
	else
		SpecStorageFrame:Show()
		if #SpecStorageFrame.children > 0 then
			SpecStorageFrame:ReleaseChildren()
		end
		self:SetupUI()
	end
end

function SpecStorage:PLAYER_SPECIALIZATION_CHANGED()
	local oldSpec = currentSpecName
	self:SetupSpecVariable()
	
	if SpecStorageFrame:IsVisible() then
		--try and reopen to same boss was just looking at if appropriate
		local selected = SpecStorageFrame.children[1].children[2].localstatus.selected
		self:RefreshDisplay()
		if (selected and oldSpec == currentSpecName) then
			SpecStorageFrame.children[1].children[2]:SelectByValue(selected)
		end
	end
end

function SpecStorage:SetupSpecVariable()
	local currentSpec = GetSpecialization()
	currentSpecName = currentSpec and select(2, GetSpecializationInfo(currentSpec))
end

function SpecStorage:CreateFrame()
	local frame = AceGUI:Create("Window")
	frame:SetTitle("Spec Storage")
	frame:SetLayout("Fill")
	frame:SetWidth(750)
	frame.frame:SetFrameStrata("MEDIUM")
	SpecStorageFrame = frame
	SpecStorageFrame:Hide()
end

function SpecStorage:SetupUI()
	if not currentSpecName then
		self:Print(L["No specialization picked"])
		return
	end

	local mainGroup = AceGUI:Create("SimpleGroup")
	mainGroup:SetFullHeight(true)
	mainGroup:SetFullWidth(true)
	mainGroup:SetLayout("Flow")

	local header = AceGUI:Create("Heading")
	header:SetText(L["Saved specs for"].." "..currentSpecName)
	header:SetFullWidth(true)	
	mainGroup:AddChild(header)

	local treeGroup = AceGUI:Create("TreeGroup")
	local tree = self:SetupTree()
	treeGroup:SetFullWidth(true)
	treeGroup:SetLayout("Fill")
	treeGroup:SetFullHeight(true)
	treeGroup:SetTree(tree)
	treeGroup:SetCallback("OnGroupSelected", function(...) SpecStorage:SelectTree(...) end)
	treeGroup:SelectByPath(1)

	mainGroup:AddChild(treeGroup)
	SpecStorageFrame:AddChild(mainGroup)
end

function SpecStorage:SetupTree()
	if not (self.db.class[currentSpecName]) then
		self.db.class[currentSpecName] = {}
	end

	local tree = {}
	local addNewGroup = {}
	addNewGroup.value = "Add"
	addNewGroup.text = L["Add New"]
	table.insert(tree, addNewGroup)

	for instance, bosses in pairs(self.db.class[currentSpecName]) do
		local temp = {}
		temp.children = {}
		for bossName, specDetails in pairs(bosses) do
			local tempA = {}
			tempA.value = bossName
			tempA.text = bossName
			table.insert(temp.children, tempA)
		end
		temp.value = instance
		temp.text = instance
		table.insert(tree, temp)
	end

	return tree
end

function SpecStorage:SelectTree(parent, _, selection)
	local selectedParent, selectedChild = ("\001"):split(selection)
	parent:ReleaseChildren()
	
	local scrollFrame = AceGUI:Create("ScrollFrame")
	scrollFrame:SetLayout("List")
	scrollFrame:SetFullWidth(true)
	
	if (selectedParent == "Add" or not selectedChild) then
		addChildNumber = 1
		self:SetupAddNewRow(scrollFrame)
		self:SetupButtonGroup(scrollFrame)
	else
		addChildNumber = 2
		self:SetupSavedList(scrollFrame, selectedParent, selectedChild)
		self:SetupAddNewRow(scrollFrame)
		self:SetupButtonGroup(scrollFrame)
	end

	parent:AddChild(scrollFrame)
end

function SpecStorage:SetupSavedList(parent, instanceName, bossName)
	self:SetupLiveGlyphs()
		
	local container = AceGUI:Create("SimpleGroup")
	container:SetFullWidth(true)
	container:SetLayout("List")	

	if not (self.db.class[currentSpecName]) then
		self.db.class[currentSpecName] = {}
	end

	local bossRow = AceGUI:Create("InlineGroup")
	bossRow:SetLayout("Flow")
	bossRow:SetTitle(instanceName..": "..bossName)
	bossRow:SetFullWidth(true)
	for i=1, 6 do
		local tmpLabel = AceGUI:Create("InteractiveLabel")
		local talentName = self.db.class[currentSpecName][instanceName][bossName]["Tier"..i.."Talent"]
		tmpLabel:SetHeight(50)
		tmpLabel:SetWidth(50)
		tmpLabel:SetImage(self.db.class[currentSpecName][instanceName][bossName]["Tier"..i.."TalentTexture"])
		tmpLabel:SetUserData("Name", talentName)
		tmpLabel:SetCallback("OnEnter", function(self, _) SpecStorage:SetupTooltip(self) end)
		tmpLabel:SetCallback("OnLeave", function() GameTooltip:Hide() end)
		tmpLabel:SetImageSize(40,40)
		if (talentName == L["Blank"]) then
			--Blanks are wildcards so always active
			self:SetActive(tmpLabel.image)
		else
			local unpicked, talentIndex = GetTalentRowSelectionInfo(i)
			if unpicked then
				--Display the talent as inactive
				self:SetInactive(tmpLabel.image)				
			else
				local name = GetTalentInfo(talentIndex)
				if (name == talentName) then
					--Correct talent
					self:SetActive(tmpLabel.image)
				else
					--Wrong talent
					self:SetInactive(tmpLabel.image)
				end
			end
		end
		bossRow:AddChild(tmpLabel)
	end
	--Wanted to put a gap between talents and glyphs
	local spacer = AceGUI:Create("Label")
	spacer:SetWidth(15)
	bossRow:AddChild(spacer)

	for i=1, 3 do
		local tmpLabel = AceGUI:Create("InteractiveLabel")
		local glyphName = self.db.class[currentSpecName][instanceName][bossName]["Glyph"..i]
		tmpLabel:SetHeight(50)
		tmpLabel:SetWidth(50)
		tmpLabel:SetImage(self.db.class[currentSpecName][instanceName][bossName]["Glyph"..i.."Texture"])
		tmpLabel:SetUserData("Name", glyphName)
		tmpLabel:SetCallback("OnEnter", function(self, _) SpecStorage:SetupTooltip(self) end)
		tmpLabel:SetCallback("OnLeave", function() GameTooltip:Hide() end)
		tmpLabel:SetImageSize(40,40)
		--Display the glyph as inactive
		self:SetInactive(tmpLabel.image)	
		for i=1, 3 do
			if (glyphName == L["Blank"]) then
				--Blanks are wildcards so always active
				self:SetActive(tmpLabel.image)
				break;
			end
			if (liveMajorGlyphs[i] == glyphName) then
				--Glyph that is meant to be used is active
				self:SetActive(tmpLabel.image)
				break;
			end
		end
		bossRow:AddChild(tmpLabel)
	end
		
	local editButton = AceGUI:Create("Button")
	editButton:SetWidth(65)
	editButton:SetText(L["Edit"])
	editButton:SetCallback("OnClick", function() SpecStorage:EditingStoredSpec(instanceName, bossName) end)
	bossRow:AddChild(editButton)

	local deleteButton = AceGUI:Create("Button")
	deleteButton:SetWidth(85)
	deleteButton:SetText(L["Delete"])
	deleteButton:SetCallback("OnClick", function() SpecStorage:DeleteStoredSpec(instanceName, bossName) end)
	bossRow:AddChild(deleteButton)

	container:AddChild(bossRow)

	parent:AddChild(container)
end

function SpecStorage:SetupAddNewRow(parent)
	local container = AceGUI:Create("InlineGroup")
	container:SetLayout("Flow")
	container:SetFullWidth(true)
	container:SetTitle(L["Add/Edit a spec"])

	local instanceName = AceGUI:Create("EditBox")
	instanceName:SetLabel(L["Instance name"])
	instanceName:SetUserData("Instance","Name")
	instanceName:SetWidth(200)
	container:AddChild(instanceName)

	local bossName = AceGUI:Create("EditBox")
	bossName:SetLabel(L["Boss name"])
	bossName:SetUserData("Boss","Name")
	bossName:SetWidth(200)
	container:AddChild(bossName)

	local talent1Dropdown = self:CreateTalentDropdown(1)
	talent1Dropdown:SetLabel(L["Tier 1 Talent"])
	talent1Dropdown:SetUserData("Tier","1")
	container:AddChild(talent1Dropdown)

	local talent2Dropdown = self:CreateTalentDropdown(2)
	talent2Dropdown:SetLabel(L["Tier 2 Talent"])
	talent2Dropdown:SetUserData("Tier","2")
	container:AddChild(talent2Dropdown)

	local talent3Dropdown = self:CreateTalentDropdown(3)
	talent3Dropdown:SetLabel(L["Tier 3 Talent"])
	talent3Dropdown:SetUserData("Tier","3")
	container:AddChild(talent3Dropdown)

	local talent4Dropdown = self:CreateTalentDropdown(4)
	talent4Dropdown:SetLabel(L["Tier 4 Talent"])
	talent4Dropdown:SetUserData("Tier","4")
	container:AddChild(talent4Dropdown)

	local talent5Dropdown = self:CreateTalentDropdown(5)
	talent5Dropdown:SetLabel(L["Tier 5 Talent"])
	talent5Dropdown:SetUserData("Tier","5")
	container:AddChild(talent5Dropdown)

	local talent6Dropdown = self:CreateTalentDropdown(6)
	talent6Dropdown:SetLabel(L["Tier 6 Talent"])
	talent6Dropdown:SetUserData("Tier","6")
	container:AddChild(talent6Dropdown)

	local glyph1Dropdown = self:CreateGlyphDropdown()
	glyph1Dropdown:SetLabel(L["Major glyph 1"])
	glyph1Dropdown:SetUserData("Glyph","1")
	container:AddChild(glyph1Dropdown)
	
	local glyph2Dropdown = self:CreateGlyphDropdown()
	glyph2Dropdown:SetLabel(L["Major glyph 2"])
	glyph2Dropdown:SetUserData("Glyph","2")
	container:AddChild(glyph2Dropdown)

	local glyph3Dropdown = self:CreateGlyphDropdown()
	glyph3Dropdown:SetLabel(L["Major glyph 3"])
	glyph3Dropdown:SetUserData("Glyph","3")
	container:AddChild(glyph3Dropdown)

	local saveButton = AceGUI:Create("Button")
	saveButton:SetText(L["Save selection"])
	saveButton:SetCallback("OnClick",function() SpecStorage:AddSelection() end)

	container:AddChild(saveButton)

	parent:AddChild(container)
end

function SpecStorage:SetupButtonGroup(parent)
	local buttonGroup = AceGUI:Create("SimpleGroup")
	buttonGroup:SetLayout("Flow")
	buttonGroup:SetFullWidth(true)

	local loadLiveButton = AceGUI:Create("Button")
	loadLiveButton:SetText(L["Load Live Spec"])
	loadLiveButton:SetCallback("OnClick", function() SpecStorage:LoadLiveSpec() end)
	buttonGroup:AddChild(loadLiveButton)

	local clearButton = AceGUI:Create("Button")
	clearButton:SetText(L["Clear Talents and Glyphs"])
	clearButton:SetCallback("OnClick", function() SpecStorage:ClearAll() end)
	buttonGroup:AddChild(clearButton)

	parent:AddChild(buttonGroup)
end

function SpecStorage:SetupNameToIndex()
	if (next(talentNameToIndex) == nil) then
		for i=1, GetNumTalents(1) do
			local name = GetTalentInfo(i)
			talentNameToIndex[name] = i
		end
	end
	
	if (next(glyphNameToIndex) == nil) then
		for i=2, 100 do
			--index = 1 is the header
			--Will never hit 100, just a large number to get it looping
			local name, glyphType = GetGlyphInfo(i)
			if glyphType == 2 then
				--Hit minor glyphs which we don't care about
				break;
			end
			glyphNameToIndex[name] = i-1
		end
	end
end

function SpecStorage:GetTalentList(tier)
	--name, texture, tier, column, selected, available = GetTalentInfo(talentIndex [, isInspect, talentGroup, inspectedUnit, classId])
	local talentList = {}
	local startPoint = 3*(tier-1)+1

	for i=startPoint, startPoint+2 do
		local name = GetTalentInfo(i)
		table.insert(talentList, name)
	end
	
	table.insert(talentList, "-"..L["Clear Selection"].."-")

	return talentList
end

function SpecStorage:GetGlyphList()
	--name, glyphType, isKnown, icon, glyphId, glyphLink = GetGlyphInfo(index)
	local glyphList = {}

	for i=2, 100 do
		--index = 1 is the header
		--Will never hit 100, just a large number to get it looping
		local name, glyphType = GetGlyphInfo(i)
		if glyphType == 2 then
			--Hit minor glyphs which we don't care about
			break;
		end
		table.insert(glyphList, name)
	end

	table.insert(glyphList, "-"..L["Clear Selection"].."-")

	return glyphList
end

function SpecStorage:CreateTalentDropdown(tier)
	local talentDropDown = AceGUI:Create("Dropdown")
	talentDropDown:SetWidth(150)
	local temp = self:GetTalentList(tier)
	talentDropDown:SetList(temp)
	talentDropDown:SetUserData("BlankIndex",#temp)
	talentDropDown:SetCallback("OnValueChanged", function(self) 
		if self:GetValue() == self:GetUserData("BlankIndex") then
			self:SetValue(0)
		end
	end)

	return talentDropDown
end

function SpecStorage:CreateGlyphDropdown()
	local glyphDropDown = AceGUI:Create("Dropdown")
	glyphDropDown:SetWidth(150)
	local temp = self:GetGlyphList()
	glyphDropDown:SetList(temp)
	glyphDropDown:SetUserData("BlankIndex",#temp)
	glyphDropDown:SetCallback("OnValueChanged", function(self) 
		if self:GetValue() == self:GetUserData("BlankIndex") then
			self:SetValue(0)
		end
	end)

	return glyphDropDown
end

function SpecStorage:AddSelection()
	local instanceName
	local bossName
	local tierTalents = {}
	local tierTalentTextures = {}
	local glyphs = {}
	local glyphTextures = {}

	for objectIndex,objectAttributes in pairs(SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children) do
		for objectAttribute,objectAttributeTable in pairs(objectAttributes) do
			if objectAttribute == "userdata" then
				for key,value in pairs(objectAttributeTable) do
					if (key == "Instance") then
						instanceName = SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:GetText()
					elseif (key == "Boss") then
						bossName = SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:GetText()
					elseif (key == "Tier") then
						local talentNumber = SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:GetValue()
						if (talentNumber and talentNumber~=0) then 
							local actualTalentNumber = 3*(value-1) + talentNumber
							local talentName, talentTexture = GetTalentInfo(actualTalentNumber)
							table.insert(tierTalents,value,talentName)
							table.insert(tierTalentTextures,value,talentTexture)
						else
							table.insert(tierTalents,value,L["Blank"])
							table.insert(tierTalentTextures,value,questionMarkTexture)
						end
					elseif (key == "Glyph") then
						local glyphNumber = SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:GetValue()
						if (glyphNumber and glyphNumber~=0) then
							-- +1 due to skipping over first (the header) when building the dropdown
							local glyphName, _, _, glyphIcon = GetGlyphInfo(glyphNumber + 1)
							table.insert(glyphs,value,glyphName)
							table.insert(glyphTextures,value,glyphIcon)
						else
							table.insert(glyphs,value,L["Blank"])
							table.insert(glyphTextures,value,questionMarkTexture)
						end
					end
				end
			end
		end
	end
	--Check all fields are filled
	if (instanceName and instanceName~="" and bossName and bossName~="") then
		if not (self.db.class[currentSpecName]) then
			self.db.class[currentSpecName] = {}
		end
		if not (self.db.class[currentSpecName][instanceName]) then
			self.db.class[currentSpecName][instanceName] = {}
		end
		if not (self.db.class[currentSpecName][instanceName][bossName]) then
			self.db.class[currentSpecName][instanceName][bossName] = {}
		end
			
		for i=1, 6 do
			self.db.class[currentSpecName][instanceName][bossName]["Tier"..i.."Talent"] = tierTalents[i]
			self.db.class[currentSpecName][instanceName][bossName]["Tier"..i.."TalentTexture"] = tierTalentTextures[i]
		end

		for i=1, 3 do
			self.db.class[currentSpecName][instanceName][bossName]["Glyph"..i] = glyphs[i]
			self.db.class[currentSpecName][instanceName][bossName]["Glyph"..i.."Texture"] = glyphTextures[i]
		end

		--refresh display
		self:RefreshDisplay()
		SpecStorageFrame.children[1].children[2]:SelectByValue(instanceName.."\001"..bossName)
	else
		self:Print(L["Please enter a instance and boss name"])
	end
end

function SpecStorage:RefreshDisplay()
	if #SpecStorageFrame.children > 0 then
		SpecStorageFrame:ReleaseChildren()
	end
	self:SetupUI()
end

function SpecStorage:EditingStoredSpec(instanceName, bossName)
	self:SetupNameToIndex()

	for objectIndex,objectAttributes in pairs(SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children) do
		for objectAttribute,objectAttributeTable in pairs(objectAttributes) do
			if objectAttribute == "userdata" then
				for key,value in pairs(objectAttributeTable) do
					if (key == "Instance") then
						SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetText(instanceName)
					elseif (key == "Boss") then
						SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetText(bossName)
					elseif (key == "Tier") then
						local tempValue = self.db.class[currentSpecName][instanceName][bossName]["Tier"..value.."Talent"]
						local indexValue = 0
						if not (tempValue == L["Blank"]) then
							indexValue = talentNameToIndex[tempValue] - (3*(value-1))
						end
						SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(indexValue)
					elseif (key == "Glyph") then
						local tempValue = self.db.class[currentSpecName][instanceName][bossName]["Glyph"..value]
						local indexValue = 0
						if not (tempValue == L["Blank"]) then
							indexValue = glyphNameToIndex[tempValue]
						end
						SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(indexValue)
					end
				end
			end
		end
	end
end

function SpecStorage:DeleteStoredSpec(instanceName, bossName)
    StaticPopupDialogs["Delete_Spec"] = {
        text = L["Are you sure you wish to delete?"],
        button1 = L["Yes"],
        button2 = L["No"],
        OnAccept = function()
			self.db.class[currentSpecName][instanceName][bossName] = nil
			if (next(self.db.class[currentSpecName][instanceName]) == nil) then
				--All bosses for the instance are now gone
				self.db.class[currentSpecName][instanceName] = nil
				--refresh display
				self:RefreshDisplay()
			else
				--refresh display
				self:RefreshDisplay()
				SpecStorageFrame.children[1].children[2]:SelectByValue(instanceName)
			end
		end,
        timeout = 10,
        whileDead = true,
        hideOnEscape = true,
        preferredIndex = 3,
    }
    StaticPopup_Show("Delete_Spec")
end

function SpecStorage:SetupTooltip(item)
	local itemName = item:GetUserData("Name")
	GameTooltip:SetOwner(item.frame, "ANCHOR_TOPRIGHT")
	GameTooltip:SetText(itemName); 
	GameTooltip:Show()
end

function SpecStorage:LoadLiveSpec()
	self:SetupNameToIndex()
	self:SetupLiveGlyphs()

	for objectIndex,objectAttributes in pairs(SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children) do
		for objectAttribute,objectAttributeTable in pairs(objectAttributes) do
			if objectAttribute == "userdata" then
				for key,value in pairs(objectAttributeTable) do
					if (key == "Tier") then
						local unpicked, talentIndex = GetTalentRowSelectionInfo(value)
						if unpicked then
							SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(0)
						else
							local indexValue = talentIndex - (3*(value-1))
							SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(indexValue)
						end
					elseif (key == "Glyph") then
						local glyphName = liveMajorGlyphs[tonumber(value)]
						if glyphName == L["Blank"] then
							SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(0)
						else
							local indexValue = glyphNameToIndex[glyphName]
							SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(indexValue)
						end
					end
				end
			end
		end
	end
end

function SpecStorage:ClearAll()
	for objectIndex,objectAttributes in pairs(SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children) do
		for objectAttribute,objectAttributeTable in pairs(objectAttributes) do
			if objectAttribute == "userdata" then
				for key,value in pairs(objectAttributeTable) do
					if (key == "Tier") then
						SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(0)
					elseif (key == "Glyph") then
						SpecStorageFrame.children[1].children[2].children[1].children[addChildNumber].children[objectIndex]:SetValue(0)
					end
				end
			end
		end
	end
end

function SpecStorage:SetupLiveGlyphs()
	local glyphCount = 1

	for i = 1, NUM_GLYPH_SLOTS do
		local enabled, glyphType, glyphTooltipIndex, glyphSpellID, icon, glyphID = GetGlyphSocketInfo(i);
		if ( enabled and glyphType == 1 ) then
			local link = GetGlyphLink(i)
			if ( link ~= "") then
				local name = GetSpellInfo(glyphSpellID)
				liveMajorGlyphs[glyphCount] = strsub(name, 10)
				glyphCount = glyphCount + 1
			else
				liveMajorGlyphs[glyphCount] = L["Blank"]
				glyphCount = glyphCount + 1
			end
		end
	end
end

function SpecStorage:SetActive(object)
	--object:SetVertexColor(0,1,0)
	object:SetDesaturated(nil)
end

function SpecStorage:SetInactive(object)
	--object:SetVertexColor(1,0,0)
	object:SetDesaturated(1)
end